#include "example.h"

#define OPT_ARG_LIST        "hd:i:"

static void usage( char *pgmName );

/********************************************************
**
** Test Module: GetLatLong.c
**
** Purpose:     Get the latitude & longitude from the
**              board.
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    TPRO_LongObj    longObj;
    TPRO_LatObj     latObj;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*
     * loop through and get the latitude and longitude
     */
    int             tidx;
    char            *tstName;
    unsigned short  tstDegrees;
    float           tstMinutes;
    for ( tidx = 0 ; tidx < 2 ; tidx++ )
    {
        /*
         * perform the appropriate get operation
         */
        if (tidx == 0)
        {
            tstName = "Latitude";
            rc = TPRO_getLatitude(hnd, &latObj);
            tstDegrees = latObj.degrees;
            tstMinutes = latObj.minutes;
        }
        else
        {
            tstName = "Longitude";
            rc = TPRO_getLongitude(hnd, &longObj);
            tstDegrees = longObj.degrees;
            tstMinutes = longObj.minutes;
        }

        if ( rc != TPRO_SUCCESS )
        {
            switch (rc)
            {
                case TPRO_HANDLE_ERR:
                    errMsg = "TPRO_HANDLE_ERR";
                    break;
                case TPRO_COMM_ERR:
                    errMsg = "TPRO_COMM_ERR";
                    break;
                case TPRO_INVALID_BOARD_TYPE_ERR:
                    errMsg = "INVALID BOARD ERROR";
                    break;
                default:
                    errMsg = "Unknown error";
                    break;
            }
            (void) printf("%s: <%s>  %s rc: <%d>.\n", 
                        tstName, fullDevName, errMsg, rc);

            ret = 1;
            break;
        }
        else
        {
            /*
             * success:  print out latitude or longitude
             */
            (void)printf("<%s> %s: <%d> degrees <%f> minutes\n", 
                        fullDevName, tstName, tstDegrees, tstMinutes);
        }
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}
